@@ -31,7 +31,7 @@ from .helpers import (
     _endpoint_from_view_func, find_package, get_debug_flag, get_env,
     get_flashed_messages, url_for,
 )
-from .json import JSONDecoder, JSONEncoder, tojson_filter
+from .json import JSONDecoder, JSONEncoder, jsonify, tojson_filter
 from .logging import create_logger, create_serving_logger
 from .routing import Map, MapAdapter, Rule
 from .sessions import SecureCookieSession, SecureCookieSessionInterface, Session
@@ -42,7 +42,10 @@ from .signals import (
 )
 from .static import PackageStatic
 from .templating import _default_template_context_processor, DispatchingJinjaLoader, Environment
-from .testing import make_test_headers_path_and_query_string, no_op_push, QuartClient
+from .testing import (
+    make_test_body_with_headers, make_test_headers_path_and_query_string, no_op_push, QuartClient,
+    sentinel,
+)
 from .typing import FilePath, ResponseReturnValue
 from .utils import ensure_coroutine, file_path_to_path
 from .wrappers import BaseRequestWebsocket, Request, Response, Websocket
@@ -308,12 +311,18 @@ class Quart(PackageStatic):
         if request is not None:
             host = request.host
             return self.url_map.bind_to_request(
-                request.scheme, host, request.method, request.path, request.query_string,
+                request.is_secure,
+                host,
+                request.method,
+                request.path,
+                request.query_string,
+                isinstance(request, self.websocket_class),
+                request.root_path,
             )
 
         if self.config['SERVER_NAME'] is not None:
             return self.url_map.bind(
-                self.config['PREFERRED_URL_SCHEME'], self.config['SERVER_NAME'],
+                self.config['PREFER_SECURE_URLS'], self.config['SERVER_NAME'],
             )
         return None
 
@@ -1373,13 +1382,13 @@ class Quart(PackageStatic):
 
         config = HyperConfig()
         config.access_log_format = "%(h)s %(r)s %(s)s %(b)s %(D)s"
-        config.access_logger = create_serving_logger()  # type: ignore
+        config.accesslog = create_serving_logger()
         config.bind = [f"{host}:{port}"]
         config.ca_certs = ca_certs
         config.certfile = certfile
         if debug is not None:
             self.debug = debug
-        config.error_logger = config.access_logger  # type: ignore
+        config.errorlog = config.accesslog
         config.keyfile = keyfile
         config.use_reloader = use_reloader
 
@@ -1405,6 +1414,11 @@ class Quart(PackageStatic):
             query_string: Optional[dict]=None,
             scheme: str='http',
             send_push_promise: Callable[[str, Headers], Awaitable[None]]=no_op_push,
+            data: Optional[AnyStr]=None,
+            form: Optional[dict]=None,
+            json: Any=sentinel,
+            root_path: str="",
+            http_version: str="1.1",
     ) -> RequestContext:
         """Create a request context for testing purposes.
 
@@ -1428,10 +1442,13 @@ class Quart(PackageStatic):
         headers, path, query_string_bytes = make_test_headers_path_and_query_string(
             self, path, headers, query_string,
         )
+        request_body, body_headers = make_test_body_with_headers(data, form, json)
+        headers.update(**body_headers)
         request = self.request_class(
-            method, scheme, path, query_string_bytes, headers, send_push_promise=send_push_promise,
+            method, scheme, path, query_string_bytes, headers, root_path, http_version,
+            send_push_promise=send_push_promise,
         )
-        request.body.set_result(b'')
+        request.body.set_result(request_body)
         return self.request_context(request)
 
     async def try_trigger_before_first_request_functions(self) -> None:
@@ -1486,14 +1503,15 @@ class Quart(PackageStatic):
             status = status_or_headers
 
         if not isinstance(value, Response):
-            response = self.response_class(  # type: ignore
-                value, timeout=self.config['RESPONSE_TIMEOUT'],
-            )
+            if isinstance(value, dict):
+                response = jsonify(value)
+            else:
+                response = self.response_class(value)  # type: ignore
         else:
             response = value
 
         if status is not None:
-            response.status_code = status  # type: ignore
+            response.status_code = int(status)  # type: ignore
 
         if headers is not None:
             response.headers.update(headers)  # type: ignore
