@@ -1,5 +1,5 @@
 import os
-from typing import AsyncGenerator, Generator, Tuple, TYPE_CHECKING, Union
+from typing import Any, AsyncGenerator, Dict, Generator, Tuple, TYPE_CHECKING, Union
 
 if TYPE_CHECKING:
     from .wrappers.response import Response  # noqa: F401
@@ -8,7 +8,9 @@ FilePath = Union[bytes, str, os.PathLike]
 
 # The possible types that are directly convertable or are a Response
 # object.
-ResponseValue = Union["Response", str, AsyncGenerator[bytes, None], Generator[bytes, None, None]]
+ResponseValue = Union[
+    "Response", str, Dict[str, Any], AsyncGenerator[bytes, None], Generator[bytes, None, None],
+]
 
 # The possible types returned by a route function.
 ResponseReturnValue = Union[
