@@ -1,6 +1,7 @@
 import asyncio
+import warnings
 from datetime import datetime, timedelta, timezone
-from http.cookies import SimpleCookie
+from http.cookies import CookieError, SimpleCookie
 from os import PathLike
 from pathlib import Path
 from typing import Callable, List, Optional, TYPE_CHECKING, Union
@@ -35,6 +36,7 @@ def create_cookie(
         domain: Optional[str]=None,
         secure: bool=False,
         httponly: bool=False,
+        samesite: str=None,
 ) -> SimpleCookie:
     """Create a Cookie given the options set
 
@@ -44,8 +46,8 @@ def create_cookie(
     cookie = SimpleCookie()
     cookie[key] = value
     cookie[key]['path'] = path
-    cookie[key]['httponly'] = httponly  # type: ignore
-    cookie[key]['secure'] = secure  # type: ignore
+    cookie[key]['httponly'] = httponly
+    cookie[key]['secure'] = secure
     if isinstance(max_age, timedelta):
         cookie[key]['max-age'] = f"{max_age.total_seconds():d}"
     if isinstance(max_age, int):
@@ -56,6 +58,14 @@ def create_cookie(
         cookie[key]['expires'] = format_date_time(expires.replace(tzinfo=timezone.utc).timestamp())
     if domain is not None:
         cookie[key]['domain'] = domain
+    if samesite is not None:
+        try:
+            cookie[key]['samesite'] = samesite
+        except CookieError:
+            warnings.warn(
+                "Samesite cookies are not supported in this Python version, "
+                "please upgrade to >= 3.8"
+            )
     return cookie
 
 
