@@ -3,6 +3,7 @@ from datetime import datetime, timedelta
 from email.utils import parsedate_to_datetime
 from hashlib import md5
 from inspect import isasyncgen
+from io import BytesIO
 from os import PathLike
 from types import TracebackType
 from typing import (
@@ -17,14 +18,16 @@ from aiofiles.threadpool import AsyncFileIO
 
 from ._base import _BaseRequestResponse, JSONMixin
 from ..datastructures import (
-    CIMultiDict, ContentRange, Headers, HeaderSet, Range, ResponseAccessControl,
-    ResponseCacheControl,
+    CIMultiDict, ContentRange, ContentSecurityPolicy, Headers, HeaderSet, Range,
+    ResponseAccessControl, ResponseCacheControl,
 )
 from ..utils import create_cookie, file_path_to_path
 
 if TYPE_CHECKING:
     from ..routing import Rule  # noqa
 
+sentinel = object()
+
 
 class ResponseBody(ABC):
     """Base class wrapper for response body data.
@@ -186,6 +189,69 @@ class FileBody(ResponseBody):
         return self.size
 
 
+class IOBody(ResponseBody):
+    """Provides an async file accessor with range setting.
+
+    The :attr:`Response.response` attribute must be async-iterable and
+    yield bytes, which this wrapper does for a file. In addition it
+    allows a range to be set on the file, thereby supporting
+    conditional requests.
+
+    Attributes:
+        buffer_size: Size in bytes to load per iteration.
+    """
+    buffer_size = 8192
+
+    def __init__(
+            self, io_stream: BytesIO, *, buffer_size: Optional[int] = None,
+    ) -> None:
+        self.io_stream = io_stream
+        self.size = io_stream.getbuffer().nbytes  # type: ignore
+        self.begin = 0
+        self.end = self.size
+        if buffer_size is not None:
+            self.buffer_size = buffer_size
+
+    async def __aenter__(self) -> "IOBody":
+        self.io_stream.seek(self.begin)
+        return self
+
+    async def __aexit__(self, exc_type: type, exc_value: BaseException, tb: TracebackType) -> None:
+        return None
+
+    def __aiter__(self) -> "IOBody":
+        return self
+
+    async def __anext__(self) -> bytes:
+        current = self.io_stream.tell()
+        if current >= self.end:
+            raise StopAsyncIteration()
+        read_size = min(self.buffer_size, self.end - current)
+        chunk = self.io_stream.read(read_size)
+
+        if chunk:
+            return chunk
+        else:
+            raise StopAsyncIteration()
+
+    async def convert_to_sequence(self) -> bytes:
+        result = bytearray()
+        async with self as response:
+            async for data in response:
+                result.extend(data)
+        return bytes(result)
+
+    async def make_conditional(
+        self, begin: int, end: Optional[int], max_partial_size: Optional[int]=None,
+    ) -> int:
+        self.begin = begin
+        self.end = self.size if end is None else end
+        if max_partial_size is not None:
+            self.end = min(self.begin + max_partial_size, self.end)
+        _raise_if_invalid_range(self.begin, self.end, self.size)
+        return self.size
+
+
 class Response(_BaseRequestResponse, JSONMixin):
     """This class represents a response.
 
@@ -209,6 +275,7 @@ class Response(_BaseRequestResponse, JSONMixin):
     data_body_class = DataBody
     file_body_class = FileBody
     implicit_sequence_conversion = True
+    io_body_class = IOBody
     iterable_body_class = IterableBody
 
     def __init__(
@@ -218,8 +285,6 @@ class Response(_BaseRequestResponse, JSONMixin):
             headers: Optional[Union[dict, CIMultiDict, Headers]]=None,
             mimetype: Optional[str]=None,
             content_type: Optional[str]=None,
-            *,
-            timeout: Optional[int]=None,
     ) -> None:
         """Create a response object.
 
@@ -236,14 +301,12 @@ class Response(_BaseRequestResponse, JSONMixin):
             headers: Headers to attach to the response.
             mimetype: Mimetype of the response.
             content_type: Content-Type header value.
-            timeout: Optional argument to specify timeout when sending
-                response data.
 
         Attributes:
             response: An iterable of the response bytes-data.
         """
         super().__init__(headers)
-        self.timeout = timeout
+        self.timeout: Union[int, None, object] = sentinel
 
         if status is None:
             status = self.default_status
@@ -348,6 +411,7 @@ class Response(_BaseRequestResponse, JSONMixin):
             domain: Optional[str]=None,
             secure: bool=False,
             httponly: bool=False,
+            samesite: str=None,
     ) -> None:
         """Set a cookie in the response headers.
 
@@ -356,7 +420,7 @@ class Response(_BaseRequestResponse, JSONMixin):
         """
         if isinstance(value, bytes):
             value = value.decode()  # type: ignore
-        cookie = create_cookie(key, value, max_age, expires, path, domain, secure, httponly)  # type: ignore  # noqa: E501
+        cookie = create_cookie(key, value, max_age, expires, path, domain, secure, httponly, samesite)  # type: ignore  # noqa: E501
         self.headers.add('Set-Cookie', cookie.output(header=''))
 
     def delete_cookie(self, key: str, path: str='/', domain: Optional[str]=None) -> None:
@@ -516,6 +580,32 @@ class Response(_BaseRequestResponse, JSONMixin):
     def content_range(self, value: ContentRange) -> None:
         self._set_or_pop_header('Content-Range', value.to_header())
 
+    @property
+    def content_security_policy(self) -> ContentSecurityPolicy:
+        def on_update(content_security_policy: ContentSecurityPolicy) -> None:
+            self.content_security_policy = content_security_policy
+
+        return ContentSecurityPolicy.from_header(
+            self.headers.get("Content-Security-Policy", ""), on_update,
+        )
+
+    @content_security_policy.setter
+    def content_security_policy(self, value: ContentSecurityPolicy) -> None:
+        self._set_or_pop_header("Content-Security-Policy", value.to_header())
+
+    @property
+    def content_security_policy_report_only(self) -> ContentSecurityPolicy:
+        def on_update(content_security_policy: ContentSecurityPolicy) -> None:
+            self.content_security_policy_report_only = content_security_policy
+
+        return ContentSecurityPolicy.from_header(
+            self.headers.get("Content-Security-Policy-Report-Only", ""), on_update,
+        )
+
+    @content_security_policy_report_only.setter
+    def content_security_policy_report_only(self, value: ContentSecurityPolicy) -> None:
+        self._set_or_pop_header("Content-Security-Policy-Report-Only", value.to_header())
+
     @property
     def content_type(self) -> Optional[str]:
         return self.headers.get('Content-Type')
